<?php

namespace Concrete\Package\JlElephpantInfo\Controller\SinglePage\Dashboard\System\Environment;

use Concrete\Core\Page\Page;
use Concrete\Core\Routing\Redirect;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Http\Request;
use Concrete\Core\Page\Controller\DashboardPageController;
use Concrete\Core\Support\Facade\Config;
use Illuminate\Support\Arr;
use Doctrine\DBAL\Exception\DriverException;
use Throwable;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

/*
 * https://documentation.concretecms.org/developers/framework/configuration-and-keyvalue-storage/storing-configuration-values
 */

class MysqlInfo extends DashboardPageController
{
    public function on_start()
    {

        if (is_callable('parent::on_start')) {
            parent::on_start();
        }
        if (empty($this->app)) {
            $this->app = Application::getFacadeApplication();
        }
        $this->fh = $this->app->make('helper/file');
        $this->set('app', $this->app);
        $this->addHeaderItem($this->some_css());

    }

    public function view()
    {
        $data = Request::post();

        if(empty($data['dot_notation'])){
            $data['dot_notation']='';
        }

        $poss_info_cats = $this->list_poss_mysql_groups();
        $poss_info_cats['JTF_SUPPORT'] = t('Addon Support');
        $this->set('poss_info_cats', $poss_info_cats);

        /*
         * See what button was clicked. Should only be one category.
         * This is just an easy way to check through them in sequence
         */
        $info_cats = [];
        foreach ($data as $ky => $val) {
            if (preg_match("/ccm\-submit\-(\/dashboard\/system\/.*)/", $ky, $m)) {
                $page = Page::getByPath($m[1]);
                return Redirect::url($page->getCollectionLink());
            }

            if (preg_match("/ccm\-submit\-/i", $ky)) {
                $cat = array_search($val, $poss_info_cats);
                if (strpos($ky, 'JTF_SUPPORT') !== false) {
                    $info_cats[] = 'JTF_SUPPORT';
                } else if ($cat !== false) {
                    $info_cats[] = $cat;
                }
            }
        }

        if (empty($info_cats)) {
            $info_cats[] = 'global';
        }

        $selected_info_group = reset($info_cats);

        $this->set('selected_info_group', $selected_info_group);

        $selected_group_config = $this->get_mysql_group($selected_info_group, $data['dot_notation']);
        ksort($selected_group_config, SORT_NATURAL+SORT_FLAG_CASE);
        $this->set('selected_group_config',$selected_group_config );

    }

    /*
	List all config_groups
	*/
    private function list_poss_mysql_groups()
    {
        $config_groups = [
            'global' => t('Global variables'),
            'session' => t('Session variables'),
            'system' => t('System variables'),
            'site-raw' => t('Site tables raw'),
            'site-create' => t('Site tables create'),
            'site-columns' => t('Site show columns'),
            'site-table-size' => t('Site table size'),
        ];
        $qst = 'SHOW TABLES FROM `information_schema`';
        $db = $this->app->make('database')->connection();
        $system_tables = array_map(
                function($a){
                    $a = reset($a);
                    return $a;
                },
            $db->fetchAll($qst));
        if(!in_array('SYSTEM_VARIABLES',$system_tables )){
            unset($config_groups['system']);
        }

        return $config_groups;
    }


    /*
	List a specific group
	*/
    private function get_mysql_group($selected_group = 'system', $notation = null)
    {

        try{
            switch ($selected_group) {
                case 'site-raw':
                case 'site':
                    return $this->specialCaseSiteRaw($notation);
                    break;
                case 'site-create':
                    return $this->specialCaseSiteCreate($notation);
                    break;
                case 'site-columns':
                    return $this->specialCaseSiteColumns($notation);
                    break;
                case 'site-table-size':
                    return $this->specialCaseSiteTableSize($notation);
                    break;


                case 'session':
                    $q = 'SELECT * FROM `information_schema`.`SESSION_VARIABLES` ORDER BY `VARIABLE_NAME`';
                    break;
                case 'system':
                    $q = 'SELECT * FROM `information_schema`.`SYSTEM_VARIABLES` ORDER BY `VARIABLE_NAME`';
                    break;

                case 'global':
                default:
                    $q = 'SELECT * FROM `information_schema`.`GLOBAL_VARIABLES` ORDER BY `VARIABLE_NAME`';
                    break;
            }

            $db = $this->app->make('database')->connection();
            $results = $db->fetchAll($q);

        } catch (DriverException $e){
            $results = $e->getMessage();
        } catch (Throwable $e){
            $results = $e->getMessage();
        }

        if (empty($results)) {
            return;
        }

        if (is_scalar($results)) {
            return [$results];
        }

        $mysql_vars = [];
        foreach ($results as $result) {
            $usable_result = [];
            if (implode(',', array_keys($result)) === 'VARIABLE_NAME,VARIABLE_VALUE') {
                $usable_result = $result['VARIABLE_VALUE'];
            } else {
                $usable_result = $result;
                unset($usable_result['VARIABLE_NAME']);
            }

            if (!empty($usable_result) && is_array($usable_result) && $notation === 'dot') {
                $usable_result = Arr::dot($usable_result);
            }

            if (!empty($result['VARIABLE_NAME'])) {
                $name = $result['VARIABLE_NAME'];
                $mysql_vars[$name] = $usable_result;
            } else {
                $mysql_vars[] = $usable_result;
            }
        }
        return $mysql_vars;
    }


    /*
     * We have different ways of showing table/column schema.
     * The raw data from  the information_schema
     */
    protected function specialCaseSiteRaw($notation = null)
    {
        $db = $this->app->make('database')->connection();
        $dbname = $db->getDatabase();

        $q = 'SELECT * FROM `information_schema`.`columns` WHERE `table_schema` = "' . $dbname . '" ORDER BY TABLE_NAME,ORDINAL_POSITION';
        $results = $db->fetchAll($q);
        if (empty($results) || !is_array($results)) {
            return;
        }
        $schema = [];
        foreach ($results as $result) {
            $usable_result = $result;
            unset($usable_result['TABLE_CATALOG']);
            unset($usable_result['TABLE_SCHEMA']);
            unset($usable_result['TABLE_NAME']);
            unset($usable_result['COLUMN_NAME']);
            unset($usable_result['ORDINAL_POSITION']);

            $table = $result['TABLE_NAME'];
            $col = $result['COLUMN_NAME'];

            if ($notation === 'dot') {
                $schema[$table.'.'.$col] = $usable_result;
            }else{
                $schema[$table][$col] = $usable_result;
            }


        }


        return $schema;
    }

    /*
     * We have different ways of showing table/column schema.
     * The create command for each table
     */
    protected function specialCaseSiteCreate($notation = null)
    {
        $db = $this->app->make('database')->connection();
        $dbname = $db->getDatabase();

        $qt = 'SHOW TABLES FROM ' . $dbname;
        $tables = $db->fetchAll($qt);
        if (empty($tables) || !is_array($tables)) {
            return;
        }
        $schema = [];
        foreach ($tables as $table_data) {
            $table_name = reset($table_data);
            $q2 = 'SHOW CREATE TABLE ' . $table_name;
            $create_cmd = $db->fetchAll($q2);
            $schema[$table_name] = $create_cmd[0]['Create Table'];
        }
        return $schema;
    }

    /*
     * We have different ways of showing table/column schema.
     * The column spec for each column
     */
    protected function specialCaseSiteColumns($notation = null)
    {
        $db = $this->app->make('database')->connection();
        $dbname = $db->getDatabase();

        $qt = 'SHOW TABLES FROM ' . $dbname;
        $tables = $db->fetchAll($qt);
        if (empty($tables) || !is_array($tables)) {
            return;
        }
        $schema = [];
        foreach ($tables as $table_data) {
            $table_name = reset($table_data);
            $q2 = 'SHOW COLUMNS FROM ' . $table_name;
            $column_data = $db->fetchAll($q2);

            if ($notation === 'dot') {
                foreach ($column_data as $col_data_item) {
                    $col_schema = $col_data_item;
                    unset($col_schema['Field']);
                    $schema[$table_name.'.'.$col_data_item['Field']] = $col_schema;
                }

            }else{
                $table_schema = [];
                foreach ($column_data as $col_data_item) {
                    $col_schema = $col_data_item;
                    unset($col_schema['Field']);
                    $table_schema[$col_data_item['Field']] = $col_schema;
                }
                $schema[$table_name] = $table_schema;
            }
        }
        return $schema;
    }

    /*
     * The size of each table
     */
    protected function specialCaseSiteTableSize($notation)
    {
        $db = $this->app->make('database')->connection();
        $dbname = $db->getDatabase();
        $qt = 'SHOW TABLES FROM ' . $dbname;
        $tables = $db->fetchAll($qt);
        if (empty($tables) || !is_array($tables)) {
            return;
        }
        $sizes = [];
        foreach ($tables as $table_data) {
            $table_name = reset($table_data);
            $q2 = 'SELECT COUNT(*) AS "size" FROM ' . $table_name;
            $size_data = $db->fetchAll($q2);
            $sizes[$table_name] = $size_data[0]['size'];
        }
        return $sizes;
    }


    /**
     * Some css to pretty up the output format
     */
    public function some_css()
    {
        ob_start();
        ?>
        <style type="text/css">
            .elephant-v9-header {
                display: none;
            }

            #info_data_notation {
                display: inline-block;
                margin-right: 30px;
            }

            .elephant-info_support #table_filter_container,
            .elephant-config_database #info_data_notation,
            .elephant-jtf_support #info_data_notation,
            .elephant-info_support #info_data_notation {
                display: none;
            }

            #mysql_info_form .ccm-dashboard-header-buttons .btn {
                margin-left: 5px;
                margin-bottom: 5px;
            }

            #mysql_info_form .config_header_menu .dropdown-menu {
                right: 0;
                left: auto;
            }

            #mysql_info_form tr > td:first-child, #mysql_info_form th {
                background: #EEE;
            }

            #mysql_info_form td.v {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-all;
            }

            #mysql_info_form pre {
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-word;
            }

            #mysql_info_form h2 a, #mysql_info_form h2 a:hover {
                text-decoration: none;
                color: inherit;
            }

            #mysql_info_form ul.dropdown-menu{
                padding-left:10px;
                padding-right:10px;
                width: 820px;
                max-width: calc(100vw - 100px);
                column-count: 3;
                column-gap: 15px;
                column-rule: 1px solid #EEE;
            }

            #mysql_info_form ul.dropdown-menu .active {
                color: #fff;
                text-decoration: none;
                background-color: #4a90e2;
            }

            #mysql_info_form ul li {
                clear: both;
            }

            #mysql_info_form ul li.info_first_extra {
                border-top: 1px solid #eee;
            }
        </style>
        <?php

        $css = ob_get_contents();
        ob_end_clean();
        return $css;
    }

}
