<?php

namespace Concrete\Package\JlElephpantInfo\Controller\SinglePage\Dashboard\System\Environment;

use Concrete\Core\Page\Page;
use Concrete\Core\Routing\Redirect;
use Concrete\Core\Support\Facade\Application;
use Concrete\Core\Http\Request;
use Concrete\Core\Page\Controller\DashboardPageController;
use Illuminate\Support\Arr;

defined('C5_EXECUTE') or die('Access Denied');

/*
 * John Liddiard (aka JohntheFish) www.c5magic.co.uk
 * This software is licensed under the terms described in the concrete5.org marketplace.
 * Please find the add-on there for the latest license copy.
 */

class SuperglobalsInfo extends DashboardPageController
{
    public function on_start()
    {

        if (is_callable('parent::on_start')) {
            parent::on_start();
        }
        if (empty($this->app)) {
            $this->app = Application::getFacadeApplication();
        }
        $this->set('app', $this->app);
        $this->addHeaderItem($this->some_css());

    }

    public function view()
    {
        $data = Request::post();

        if(empty($data['dot_notation'])){
            $data['dot_notation']='';
        }

        $poss_info_cats = $this->list_poss_superglobal_groups();

        $poss_info_cats['JTF_SUPPORT'] = t('Addon Support');
        $this->set('poss_info_cats', $poss_info_cats);

        /*
         * See what button was clicked. Should only be one category.
         * This is just an easy way to check through them in sequence
         */
        $info_cats = [];
        foreach ($data as $ky => $val) {
            if(preg_match("/ccm\-submit\-(\/dashboard\/system\/.*)/", $ky, $m)){
                $page = Page::getByPath($m[1]);
                return Redirect::url($page->getCollectionLink());
            }
            if (preg_match("/ccm\-submit\-/i", $ky)) {
                $cat = array_search($val, $poss_info_cats);
                if(strpos($ky,'JTF_SUPPORT')!==false){
                    $info_cats[] = 'JTF_SUPPORT';
                }else if ($cat !== false) {
                    $info_cats[] = $cat;
                }
            }
        }

        if (empty($info_cats)) {
            $info_cats[] = '_SERVER';
        }

        $selected_superglobal_group = reset($info_cats);

        $this->set('selected_superglobal_group', $selected_superglobal_group);
        $this->set('selected_group_constants', $this->get_superglobal_group($selected_superglobal_group));

    }

    /*
	List all superglobal_groups
	*/
    private function list_poss_superglobal_groups(){
        $base = '$GLOBALS $_SERVER $_GET $_POST $_FILES $_COOKIE $_SESSION $_REQUEST $_ENV';
        $list = preg_split("/\s+/",$base );
        $list2 = array_map(function($a){return substr($a,1);}, $list);
        return array_combine($list2,$list2);
    }

    private function get_superglobal_group($selected_superglobal_group){

        switch($selected_superglobal_group) {
            case '$GLOBALS':
            case 'GLOBALS':
                return $GLOBALS;
                break;
            case '$_SERVER':
            case '_SERVER':
                return $_SERVER;
                break;
            case '$_GET':
            case '_GET':
            return $_GET;
                break;
            case '$_POST':
            case '_POST':
            return $_POST;
                break;
            case '$_FILES':
            case '_FILES':
            return $_FILES;
                break;
            case '$_COOKIE':
            case '_COOKIE':
            return $_COOKIE;
                break;
            case '$_SESSION':
            case '_SESSION':
            return $_SESSION;
                break;
            case '$_REQUEST':
            case '_REQUEST':
            return $_REQUEST;
                break;
            case '$_ENV':
            case '_ENV':
            default:
                return $_ENV;
                break;
        }
    }

    /**
     * Some css to pretty up the output format
     */
    public function some_css()
    {
        ob_start();
        ?>
        <style type="text/css">
            .elephant-v9-header{
                display:none;
            }
            #info_data_notation{
                display:inline-block;
                margin-right:30px;
            }
            .elephant-info_support #table_filter_container,
            .elephant-jtf_support #info_data_notation,
            .elephant-info_support #info_data_notation{
                display:none;
            }

            #superglobals_info_form .ccm-dashboard-header-buttons .btn{
                margin-left:5px;
                margin-bottom:5px;
            }
            #superglobals_info_form .superglobals_header_menu .dropdown-menu {
                right:0;
                left:auto;
            }
            #superglobals_info_form tr > td:first-child, #superglobals_info_form th
            {
                background: #EEE;
            }
            #superglobals_info_form td.v{
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-all;
            }
            #superglobals_info_form pre{
                word-wrap: break-word;
                white-space: pre-wrap;
                word-break: break-word;
            }
            #superglobals_info_form h2 a, #superglobals_info_form h2 a:hover{
                text-decoration:none;
                color: inherit;
            }
            #superglobals_info_form ul.dropdown-menu{
                padding-left:10px;
                padding-right:10px;
                width: 820px;
                max-width: calc( 100vw - 100px );
                column-count:3;
                column-gap:15px;
                column-rule:1px solid #EEE;
            }
            #superglobals_info_form ul.dropdown-menu .active {
                color: #fff;
                text-decoration: none;
                background-color: #4a90e2;
            }

            #superglobals_info_form ul li{
                clear:both;
            }
            #superglobals_info_form ul li.info_first_extra{
                border-top:1px solid #eee;
            }
            #superglobals_info_form .recursive-table{
                max-width:96vw;

            }
            #superglobals_info_form .recursive-table td{
                word-break: break-all;
            }
            #superglobals_info_form .recursive-table tr>td:first-child{
                width:40%;
            }
            #superglobals_info_form .recursive-table tr>td:last-child{
                width:60%;
            }
            #superglobals_info_form.recursive-table td pre{
                word-break: break-all;
                white-space: pre-wrap;
            }


        </style>
        <?php

        $css = ob_get_contents();
        ob_end_clean();
        return $css;
    }

}
